from standards import standard_arrays
from standards import calculate_Ca
from standards import calculate_fb

def fb(variables, constants):
    
    """Calculates the fraction of DNA bound to BlcR for different BlcR
    concentrations for different GHB concentrations.
    Parameters:
    ----------
    variables: list
        A list containing the variables: log of mininimum of BlcR,
        log of maximum of BlcR, log of minimum of GHB, log of maximum of GHB, 
        log of minimum Kd BlcR and DNA, log of maximum Kd BlcR and DNA,
        log of minimum Kd BlcR and GHB, log of maximum Kd BlcR and GHB.
    constants: list
        A list containg the constants: Surface of electrode,
        Volume between the electrodes, Hill coefficient, Kd of BlcR to DNA,
        Kd of BlcR to GHB, Kd of BlcR to DNA when GHB is bound,
        Max surface coverage with DNA, DNA total, 
        concentration of DNA between electrodes,cutoff,GHB_drink, Max_BlcR.
    
    Returns
    ----------
    fb1: list
        List containing values of fraction bound for different GHB and BlcR concentrations. 
        hill coefficient=1
    fb2: list
        List containing values of fraction bound for different GHB and BlcR concentrations. 
        hill coefficient=2
    Ca_0: list
        List containing range of concentrations of BlcR
    C_GHB_00: list
        List containing the range of concentrations of GHB
    """
    
    ## load constants
    Kd_BlcR_DNA=constants[3]
    Kd_BlcR_GHB=constants[4]
    ## load variables
    log_min_Ca=variables[0]
    log_max_Ca=variables[1]
    log_min_GHB=variables[2]
    log_max_GHB=variables[3]
    log_min_Kd_BlcR_DNA=variables[4]
    log_max_Kd_BlcR_DNA=variables[5]
    log_min_Kd_BlcR_GHB=variables[6]
    log_max_Kd_BlcR_GHB=variables[7]
    ## load arrays
    arrays=standard_arrays(log_min_Ca,log_max_Ca,log_min_GHB,log_max_GHB,log_min_Kd_BlcR_DNA,log_max_Kd_BlcR_DNA,log_min_Kd_BlcR_GHB,log_max_Kd_BlcR_GHB)
    Ca_0=arrays[2]
    C_GHB_00=arrays[3]
    fb1=arrays[4]
    fb2=arrays[5]
    ## first we calculate the fraction bound for different GHB and BlcR concentrations
    
    for i in range(len(C_GHB_00)):
        C_GHB_0=C_GHB_00[i]
        Ca=calculate_Ca(Kd_BlcR_DNA,Kd_BlcR_GHB,C_GHB_0,Ca_0)
        fb1[i]=calculate_fb(Ca,Kd_BlcR_DNA,1)
        fb2[i]=calculate_fb(Ca,Kd_BlcR_DNA,3.4)
    gain=fb1[0]*fb1[0]/(fb1[-1])
    
    return fb1,fb2,Ca_0,C_GHB_00

