import math
from standards import calculate_Ca
from standards import calculate_fb
import numpy as np
from real_range_BlcR import find_range_BlcR

def signal_to_noise_max(variables, constants, Kd_BlcR_DNA=(395*10**-9)**2,Kd_BlcR_GHB=700*10**-9):
    """Calculates a part of the noise and compares it to the signal. The noise
    Described here is the loose BlcR not bound to the sensor which can cause
    noise by floating between the sensor and a larger unbound BlcR can create other
    problems like aggregation and unspecified binding. The output will be the
    Ratio of bound to unbound BlcR between the plates.
    Parameters:
    ----------
    variables: list
        A list containing the variables: log of mininimum of BlcR,
        log of maximum of BlcR, log of minimum of GHB, log of maximum of GHB, 
        log of minimum Kd BlcR and DNA, log of maximum Kd BlcR and DNA,
        log of minimum Kd BlcR and GHB, log of maximum Kd BlcR and GHB.
    constants: list
         A list containg the constants: Surface of electrode,
        Volume between the electrodes, Hill coefficient, Kd of BlcR to DNA,
        Kd of BlcR to GHB, Kd of BlcR to DNA when GHB is bound,
        Max surface coverage with DNA, DNA total, 
        concentration of DNA between electrodes,cutoff,GHB_drink, Max_BlcR.
    Kd_BlcR_DNA: float
        The Kd between BlcR and DNA
    Kd_BlcR_GHB: float
        The Kd between BlcR and DNA
    Returns
    ----------
    Ca_0: list
        List containing the BlcR concentrations 
    Frac_cap: list
        List containing the bound/unbound ratio for a range of BlcR concentrations
    lowerbound:
        The lowest usable BlcR concentration
    """
    ## load constants
    Volume=constants[1]
    Hill_coefficient=constants[2]
    #Kd_BlcR_DNA=constants[3]
    #Kd_BlcR_GHB=constants[4]
    DNA_total=constants[7]
    ## load arrays
    Ca_00,k1,k2, upperbound, lowerbound=find_range_BlcR(variables, constants,Kd_BlcR_DNA,Kd_BlcR_GHB)
    Ca_0=np.logspace(math.log10(lowerbound),math.log10(upperbound),100)
    #C_GHB_00=arrays[3]
    ## The signal to noise is calculated for the steady state low GHB concentration
    C_GHB_0=constants[11]
    ## code  
    ## calculate BlcR not bound to GHB
    Ca=calculate_Ca(Kd_BlcR_DNA,Kd_BlcR_GHB,C_GHB_0,Ca_0)
    ## calculate fraction of DNA bound
    fb3=calculate_fb(Ca,Kd_BlcR_DNA,Hill_coefficient)
    ## calculate amount of BlcR bound between electrodes
    Delta_BlcR=2*fb3*DNA_total
    ## calculate amount of BlcR not bound between electrodes
    BlcR_molecules=Ca_0*Volume
    ## calculate ratio of BlcR between sensors that is bound and not bound
    Frac_cap=Delta_BlcR/(BlcR_molecules)

    return Frac_cap, Ca_0, lowerbound



