from standards import standard_arrays
from standards import calculate_Ca
from standards import calculate_fb

def signal_to_noise(variables, constants,C_GHB_0=5*10**-5):
    
    """Calculates a part of the noise and compares it to the signal. The noise
    Described here is the loose BlcR not bound to the sensor which can cause
    noise by floating between the sensor and a larger unbound BlcR can create other
    problems like aggregation and unspecified binding. The output will be the
    Ratio of bound to unbound BlcR between the plates.
    Parameters:
    ----------
    variables: list
        A list containing the variables: log of mininimum of BlcR,
        log of maximum of BlcR, log of minimum of GHB, log of maximum of GHB, 
        log of minimum Kd BlcR and DNA, log of maximum Kd BlcR and DNA,
        log of minimum Kd BlcR and GHB, log of maximum Kd BlcR and GHB.
    constants: list
         A list containg the constants: Surface of electrode,
        Volume between the electrodes, Hill coefficient, Kd of BlcR to DNA,
        Kd of BlcR to GHB, Kd of BlcR to DNA when GHB is bound,
        Max surface coverage with DNA, DNA total, 
        concentration of DNA between electrodes,cutoff,GHB_drink, Max_BlcR.
    C_GHB_0: float
        The concentration of GHB 
    Returns
    ----------
    Ca_0: list
        List containing the BlcR concentrations 
    Frac_cap: list
        List containing the bound/unbound ratio for a range of BlcR concentrations
    """
    ## load constants
    Volume=constants[1]
    Hill_coefficient=constants[2]
    Kd_BlcR_DNA_0=[(390*10**-9)**2,(39*10**-9)**2,(3.9*10**-9)**2] ##constants[3]
    Kd_BlcR_GHB=700*10**-9
    DNA_total=constants[7]
    ## load variables
    log_min_Ca=variables[0]
    log_max_Ca=variables[1]
    log_min_GHB=variables[2]
    log_max_GHB=variables[3]
    log_min_Kd_BlcR_DNA=variables[4]
    log_max_Kd_BlcR_DNA=variables[5]
    log_min_Kd_BlcR_GHB=variables[6]
    log_max_Kd_BlcR_GHB=variables[7]
    ## load arrays
    arrays=standard_arrays(log_min_Ca,log_max_Ca,log_min_GHB,log_max_GHB,log_min_Kd_BlcR_DNA,log_max_Kd_BlcR_DNA,log_min_Kd_BlcR_GHB,log_max_Kd_BlcR_GHB)
    Ca_0=arrays[2]
    Frac_cap=arrays[6]
    for i in range(3):
        Kd_BlcR_DNA=Kd_BlcR_DNA_0[i]
        ## calculate BlcR not bound to GHB
        Ca=calculate_Ca(Kd_BlcR_DNA,Kd_BlcR_GHB,C_GHB_0,Ca_0)
        ## calculate fraction of DNA bound
        fb3=calculate_fb(Ca,Kd_BlcR_DNA,Hill_coefficient)
        ## calculate amount of BlcR bound between electrodes
        Delta_BlcR=2*fb3*DNA_total
        ## calculate amount of BlcR not bound between electrodes
        BlcR_molecules=Ca_0*Volume
        ## calculate ratio of BlcR between sensors that is bound and not bound
        Frac_cap[i]=Delta_BlcR/(BlcR_molecules)
    return Frac_cap, Ca_0

