# -*- coding: utf-8 -*-
"""
Created on Mon Jul  4 12:26:49 2022

@author: mauri
"""
import numpy as np

def standard_constants(): 
## constants
    surface=2*10**-5 #m^2, surface area of the electrode
    Volume=surface*50*10**-6 #m^3,volume between electrodes
    Hill_coefficient=1.79 ## Hill coefficient,depends on wet lab data 
    Kd_BlcR_DNA=(395*10**-9)**2 ##dissociation constant of BlcR to DNA in M
    Kd_BlcR_GHB=700*10**-9 ##dissociation constant of BlcR to GHB  in M
    Kd_BlcR_DNA_with_GHB=1 ##dissociation constant of BlcR to DNA when GHB is bound in M, we assume there is no binding anymore
    Max_surface_coverage_DNA=3*10**-7 #The maximum surface coverage of the electrode with DNA in mol/m^2
    DNA_total=surface*Max_surface_coverage_DNA #The total amount of DNA on the electrode in mol
    C_DNA_0= DNA_total/Volume ##concentration of DNA between the electrodes
    Cutoff=5*10**-5 #The reccomended cutoff value for GHB detection. Below this value, the GHB can be endogenously present in drinks.
    GHB_drink=5*10**-2 #The average concentration of GHB in a spiked drink
    Max_BlcR=7.9*10**-6 #The maximum concentration of BlcR possible before aggregation occurs.
    constants=np.array([surface,Volume,Hill_coefficient,Kd_BlcR_DNA,Kd_BlcR_GHB,Kd_BlcR_DNA_with_GHB,Max_surface_coverage_DNA,
                        DNA_total,C_DNA_0,Cutoff,GHB_drink,Max_BlcR])
    
    return constants
## variables
def standard_variables():   
    ## Here we determine the ranges of parameters we are going to use, Ca is the BlcR concentration 
    log_min_Ca= -9
    log_max_Ca=-1
    log_min_GHB=-7
    log_max_GHB=-1.3 
    log_min_Kd_BlcR_DNA=-9.3
    log_max_Kd_BlcR_DNA=-6.4
    log_min_Kd_BlcR_GHB=-9.15
    log_max_Kd_BlcR_GHB=-3.15
    variables=np.array([log_min_Ca,log_max_Ca,log_min_GHB,log_max_GHB,log_min_Kd_BlcR_DNA,log_max_Kd_BlcR_DNA,
                        log_min_Kd_BlcR_GHB,log_max_Kd_BlcR_GHB])
    return variables
## creating arrays
def standard_arrays(log_min_Ca,log_max_Ca,log_min_GHB,log_max_GHB,log_min_Kd_BlcR_DNA,log_max_Kd_BlcR_DNA,log_min_Kd_BlcR_GHB,log_max_Kd_BlcR_GHB):
    ## here we are already creating arrays that can be used in other files
    rows =10
    columns=100
    kk=[0]*10000
    kk2=[0]*10000
    Ca_0=np.logspace(log_min_Ca,log_max_Ca,1000)
    C_GHB_00=np.logspace(log_min_GHB,log_max_GHB,10)
    fb1 = [[0 for x in range(columns)] for x in range(rows)]
    fb2 = [[0 for x in range(columns)] for x in range(rows)]
    Frac_cap =[[0 for x in range(columns)] for x in range(3)]
    Kd_BlcR_DNA_0=np.logspace((log_min_Kd_BlcR_DNA),(log_max_Kd_BlcR_DNA),10000)
    Kd_BlcR_GHB_0=np.logspace((log_min_Kd_BlcR_GHB),(log_max_Kd_BlcR_GHB),10000)
    arrays=np.array([kk,kk2,Ca_0,C_GHB_00,fb1,fb2,Frac_cap,Kd_BlcR_DNA_0,Kd_BlcR_GHB_0])
    return arrays


def calculate_Ca(Kd_BlcR_DNA,Kd_BlcR_GHB,C_GHB_0,Ca_0): 
     
    """Calculates the amount of available BlcR at a given concentration of BlcR and GHB
    Parameters:
    ----------
    Kd_BlcR_DNA: float
        Kd value between BlcR and DNA
    Kd_BlcR_GHB: float
        Kd value between BlcR and GHB
    C_GHB_0: float
        Concentration of GHB
    Ca_0: list
        List containg range of BlcR concentrations
    Returns
    ----------
    Ca: list
        A list containg the left over BlcR concentrations after GHB has bound
    """
    
    Ca=Ca_0-(C_GHB_0+Ca_0+Kd_BlcR_GHB-(Ca_0**2-2*C_GHB_0*Ca_0+2*Ca_0*Kd_BlcR_GHB+C_GHB_0**2+Kd_BlcR_GHB**2+2*C_GHB_0*Kd_BlcR_GHB)**0.5)/2
    return Ca

def calculate_fb(Ca,Kd_BlcR_DNA,Hill_coefficient):
     
     """Calculates the fraction of DNA bound with BlcR
     Parameters:
     ----------
     Kd_BlcR_DNA: float
         Kd value between BlcR and DNA
     Hill_coefficient: float
         Hill coefficient
     Ca: list
         List containg range of BlcR left over after GHB binding
     Returns
     ----------
     fb: list
         A list containg the fraction of DNA bound for different BlcR concentrations
     """
    
     fb=1/(1+(Kd_BlcR_DNA/(Ca)**2)**Hill_coefficient)
     return fb
 



