from standards import standard_arrays
from standards import calculate_Ca
from standards import calculate_fb

def vary_kd_BlcR_DNA(variables, constants,C_GHB_0=5*10**-5):
    
    """Calculates the concentration of BlcR needed to bind 99% of the DNA strand 
    for a range of Kd's between BlcR and DNA
    Parameters:
    ----------
    variables: list
        A list containing the variables: log of mininimum of BlcR,
        log of maximum of BlcR, log of minimum of GHB, log of maximum of GHB, 
        log of minimum Kd BlcR and DNA, log of maximum Kd BlcR and DNA,
        log of minimum Kd BlcR and GHB, log of maximum Kd BlcR and GHB.
    constants: list
         A list containg the constants: Surface of electrode,
        Volume between the electrodes, Hill coefficient, Kd of BlcR to DNA,
        Kd of BlcR to GHB, Kd of BlcR to DNA when GHB is bound,
        Max surface coverage with DNA, DNA total, 
        concentration of DNA between electrodes,cutoff,GHB_drink, Max_BlcR.
    C_GHB_0: float
        The concentration of GHB 
    Returns
    ----------
    kk: list
        List containing the BlcR concentrations needed to bind 99% of DNA.
       
    Kd_BlcR_DNA_0: list
        List containing the Kd values of BlcR to DNA that are used.
    """
    ## load constants
    Hill_coefficient=constants[2]
    Kd_BlcR_DNA=constants[3]
    Kd_BlcR_GHB=constants[4]
    ## load variables
    log_min_Ca=variables[0]
    log_max_Ca=variables[1]
    log_min_GHB=variables[2]
    log_max_GHB=variables[3]
    log_min_Kd_BlcR_DNA=variables[4]
    log_max_Kd_BlcR_DNA=variables[5]
    log_min_Kd_BlcR_GHB=variables[6]
    log_max_Kd_BlcR_GHB=variables[7]
    ## load arrays
    arrays=standard_arrays(log_min_Ca,log_max_Ca,log_min_GHB,log_max_GHB,log_min_Kd_BlcR_DNA,log_max_Kd_BlcR_DNA,log_min_Kd_BlcR_GHB,log_max_Kd_BlcR_GHB)
    kk=arrays[0]
    Kd_BlcR_DNA_0=arrays[7]
    Ca_0=arrays[2]
    ## first we calculate the fraction bound for different Kd values of BlcR and DNA
    ## for every Kd value, we store the lowest BlcR concentration for which 99% of DNA is bound
    for i in range(len(Kd_BlcR_DNA_0)):
        Kd_BlcR_DNA=(Kd_BlcR_DNA_0[i])**2
        Ca=calculate_Ca(Kd_BlcR_DNA,Kd_BlcR_GHB,C_GHB_0,Ca_0)
        fb=calculate_fb(Ca,Kd_BlcR_DNA,Hill_coefficient)
        k=[ n for n,j in enumerate(fb) if j>0.99 ][0]
        kk[i]=Ca_0[k]
    return kk,Kd_BlcR_DNA_0
